#!/bin/bash
# ffoo test harness
# See LICENSE file for copyright and license details.

. test/include/common.sh

__tf_collect_if_needed() {
    #
    # Collect artifact if exit status suggests it
    #
    # Use test exit status $1 to help decide if artifacts are
    # needed, and collect them if so.
    #
    # If TF_COLLECT is set to "always", collect regardless of
    # the status.  Otherwise, collect unless status is 0 or 1
    # (pass or bailout); in that case do nothing.
    #
    local tes=$1    # test exit status
    local will      # should we collect artifacts?
    case "$TF_COLLECT:$tes" in
        always:*)   will=true ;;
        *:0)        will=false ;;
        *:2)        will=false ;;
        *)          will=true ;;
    esac
    $will || return 0
    mkdir -p "$artifact_dir/$stamp"
    cp -r $tmpdir/* "$artifact_dir/$stamp"
}

tf_enum_tests() {
    #
    # List what looks like test; relative to test/tests
    #
    find -L \
        test/tests \
        -mindepth 2 \
        -maxdepth 2 \
        -type f \
        -perm /111 \
        -name TF_RUN \
      | cut -d/ -f3
}

tf_run_tests() {
    #
    # Discover and run tests
    #
    local artifact_dir="$TF_PROJDIR/test/artifacts"
    local es=0          # overall exit status
    local tmpdir=""     # test temporary dir
    local tname=""      # test name
    local tes=0         # test result
    local stamp=""      # test stamp to use as artifact name
    tf_think "# version: $(ffoom --version-semver)"
    tf_think ""
    es=0
    for tname in $(tf_enum_tests | grep -e "$TF_FILTER_TEST");
    do
        tf_think "... $tname"
        tmpdir=$(mktemp -d)
        stamp=$(date +artifacts-$tname-%Y%m%d-%H%M%S)
        pushd $tmpdir >/dev/null
            cp -r "$TF_PROJDIR/test/tests/$tname/"* .
            TF_TNAME="$tname" ./TF_RUN
            tes=$?
            __tf_collect_if_needed $tes
            test $tes -gt $es && es=$tes
        popd >/dev/null
        rm -rf $tmpdir
        if test $tes -eq 0;
        then
            tf_think "''' $tname ($tes)"
        else
            tf_warn "??? $tname ($tes)"
        fi
    done
    return $es
}
